<?php
/*
Plugin Name: Woocommerce IntelliPay Payments
Description: IntelliPay gateway for WooCommerce
Version: 1.3.3
Domain Path: /languages/
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


function init_intellipay_payment_gateway() {
    if ( ! class_exists( 'WC_Payment_Gateway' ) ) return;

    load_plugin_textdomain( 'intellipay-wc', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );


    class WC_Gateway_IntelliPay_Payment_Gateway extends WC_Payment_Gateway {
        

        var $notify_url;

        function __construct() {

            global $woocommerce;

            $this->id = 'intellipay';

            // URL to logo
            if($this->get_option( 'intellipay_logo' ) != 'hide'){
                $this->icon = ($this->get_option( 'intellipay_logo' ) === 'dark') ? plugins_url( 'assets/images/intellipay-dark.png', __FILE__ ) : plugins_url( 'assets/images/intellipay-light.png', __FILE__ );
            }

            // Direct integration, no
            $this->has_fields = false;

            $this->notify_url = add_query_arg( 'wc-api', 'WC_Gateway_IntelliPay_Payment_Gateway', home_url( '/' ) );

            // Title of the payment method shown on the admin page
            $this->method_title			= __( 'IntelliPay', 'intellipay-wc' );
            $this->method_description	= __( 'IntelliPay Payment method', 'intellipay-wc' );

            // Load settings fields:
            $this->init_form_fields();
            $this->init_settings();

            // Define user set variables
            $this->title		=  'IntelliPay';// $this->get_option( 'title' ); Decided to go with hard coding instead of giving the merchant the option.

            //Could make a call to see if we should show or not.
            $this->description	= __( 'Additional processing fees may apply.', 'intellipay-wc' );

            $this->merchant_key	= $this->get_option( 'intellipay_merchant_key' );
            $this->api_key		= $this->get_option( 'intellipay_api_key' );
            $this->test_mode	= $this->get_option( 'intellipay_test_mode' );
            

            // Actions
            add_action( 'woocommerce_receipt_' . $this->id							, array( $this, 'receipt_page' ) );
            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id	, array( $this, 'process_admin_options' ) );

            // Payment listener/API hook
            add_action( 'woocommerce_api_wc_gateway_intellipay_payment_gateway', array( $this, 'check_notify_response' ) );
            
        }

        function init_form_fields() {
            $this->form_fields = array(
                'enabled' => array(
                    'title'			=> __( 'Enable/Disable', 'woocommerce' ),
                    'type'			=> 'checkbox',
                    'label'			=> __( 'Enable IntelliPay', 'intellipay-wc' ),
                    'default'		=> 'yes'
                ),
                // 'title' => array( Decided to go with hard coding instead of giving the merchant the option.
                //     'title'			=> __( 'Title', 'woocommerce' ),
                //     'type'			=> 'text',
                //     'description'	=> __( 'This controls the title which the user sees during checkout.', 'woocommerce' ),
                //     'default'		=> __( 'IntelliPay', 'intellipay-wc' ),
                //     'desc_tip'		=> true,
                // ),

                'intellipay_merchant_key' => array(
                    'title'			=> __( 'Merchant Key', 'woocommerce' ),
                    'type'			=> 'text',
                    'description' => __( 'This controls the Merchant Key by IntelliPay.', 'intellipay-wc' ),
                    'desc_tip'      => true,
                    'default'		=> ''
                ),
                'intellipay_api_key' => array(
                    'title'			=> __( 'API Key', 'woocommerce' ),
                    'type'			=> 'text',
                    'default'		=> '',
                    'desc_tip'      => true,
                    'description' => __( 'This controls the API Key by IntelliPay.', 'intellipay-wc' ),
                ),
                'intellipay_logo' => array(
                    'title'			=> __( 'IntelliPay Logo', 'woocommerce' ),
                    'type'			=> 'select',
                    'description' => __( 'This controls the logo which the user sees during checkout.', 'intellipay-wc' ),
                    'desc_tip'      => true,
                    'default'		=> 'hide',
                    'options'     => array(
                        'dark'	=> __( 'Dark Logo', 'intellipay-wc' ),
                        'light'	=> __( 'Light Logo', 'intellipay-wc' ),
                        'hide'	=> __( 'Hide Logo', 'intellipay-wc' ),
                    )
                ),
                'intellipay_test_mode' => array(
                    'title'			=> __( 'Test mode', 'woocommerce' ),
                    'type'			=> 'checkbox',
                    'label'			=> __( 'Enable test mode', 'intellipay-wc' ),
                    'desc_tip'      => true,
                    'description'	=> __( 'Ensure that you have test mode set on your merchant account as well.', 'woocommerce' ),
                    'default'		=> 'no',
                ),
            );
        }


        public function admin_options() { ?>
            <h3><?php _e( 'IntelliPay', 'intellipay-wc' ); ?></h3>
            <p><?php _e( 'IntelliPay works by sending the user to through intelliPay Lightbox to enter their payment information.', 'intellipay-wc' ); ?></p>
            <p>You can get your settings by reaching out to support at:
                <br/>
                Email: <a href="mailto:support@intellipay.com"> support@intellipay.com</a>
                <br/>
                Phone: <a href="tel:855-872-6632">855-872-6632</a>
            </p>

            <table class="form-table">
                <?php
                // Generate the HTML For the settings form.
                $this->generate_settings_html();
                ?>
            </table>

            <?php
        }


        //This is what takes you from the main checkout page and goes to the receipt page to finish the payment processing.
        function process_payment( $order_id ) {

            $order = wc_get_order( $order_id );
            $redirect = add_query_arg( 'order-pay', $order->get_id(), $order->get_checkout_payment_url( true ) );

            return array(
                'result' 	=> 'success',
                'redirect'	=> $redirect
            );
        }

        function receipt_page( $order_id ) {
            global $woocommerce;

            $order = wc_get_order( $order_id );    

            $accept_url	 = $this->notify_url;
            $decline_url = $this->notify_url;
   

            
            //Custom Params being passed in
            $params = [
                'customerid' => $order->get_id(),
                // 'email' => $order->get_billing_email(),
            ];
            
            // Lightbox Documentation URL https://intellipay.com/wp-content/uploads/2021/06/LightBox-Terminal-Quick-Start-6-14-21.pdf
            $urlSubDomain = ($this->test_mode === 'yes') ? 'test' : 'secure';

            $intelliPayLightBoxUrl = 'https://'.$urlSubDomain.'.cpteller.com/api/custapi.cfc?method=autoterminal';
            
            // $result = file_get_contents(
            //     $intelliPayLightBoxUrl,
            //     false,
            //     stream_context_create(
            //         [
            //             'ssl' => [
            //                 'verify_peer' => ($this->test_mode === 'yes') ? false : true,
            //                 'verify_peer_name' => ($this->test_mode === 'yes') ? false : true,
            //                 'allow_self_signed' => ($this->test_mode === 'yes') ? true : false,
            //             ],
            //             'http' => [
            //                 'header' =>"Content-type: application/x-www-form-urlencoded\r\n",
            //                 'method' => 'POST',
            //                 'content' => http_build_query(
            //                     [

            //                         'merchantkey' => trim($this->merchant_key),
            //                         'apikey' => trim($this->api_key)
            //                     ]
            //                 )
            //             ]
            //         ]
            //     )
            // );

            // if ($result === FALSE) { /* Handle error */ echo 'Error loading lightbox. Mostly likely cause is with your SSL Cert.';}


            $fields = array(
                'merchantkey' => trim($this->merchant_key),
                'apikey' => trim($this->api_key)
            );
            $fields_string = '';
            foreach($fields as $key => $value){
                $fields_string .= $key.'='.$value.'&';
            }
            rtrim($fields_string, '&');
            
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $intelliPayLightBoxUrl);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);            
            
            $result = curl_exec($ch);


            if(curl_exec($ch) === false){
                echo 'Request Error:' . curl_error($ch);
            }

            curl_close($ch);


            
            //This is what outputs the lightbox
            echo $result;

            ?>

            <!-- Pre Populating Values in the Lightbox -->
            <input type="hidden" class="ipayfield" data-ipayname="account" value="<?php echo $order->get_id(); ?>"/>
            <input type="hidden" class="ipayfield" data-ipayname="email" value="<?php echo $order->get_billing_email(); ?>"/>
            <input type="hidden" class="ipayfield" data-ipayname="firstname" value="<?php echo $order->get_billing_first_name(); ?>">
            <input type="hidden" class="ipayfield" data-ipayname="lastname" value="<?php echo $order->get_billing_last_name(); ?>">
            <input type="hidden" class="ipayfield" data-ipayname="zipcode" value="<?php echo $order->get_billing_postcode(); ?>">
            <input type="hidden" class="ipayfield" data-ipayname="invoice" value="<?php echo $order->get_id(); ?>">
            <input type="hidden" class="ipayfield" data-ipayname="amount" value="<?php echo $order->get_total(); ?>"/>
            

            
            <button id="intelliPayButton" data-ipayname="submit" class="ipayfield" type="button">Pay Via IntelliPay</button>
            
        
            <script type="text/javascript">

                function isIntelliPayEvent(e) {
                    var domain = e.origin.substring(
                        e.origin.indexOf(".") + 1, 
                        e.origin.lastIndexOf(".")
                    );
                    return domain === 'cpteller'
                };

                function defaultLoadOptions(){
                
                    // intellipay.disable("email");
                    // intellipay.disable("account");

                    //Uncomment if you want a delay before launching.
                    // setTimeout(function(){
                        openIntelliPayLightBox();
                    // },2000);   

                }

                function openIntelliPayLightBox(){
                    document.getElementById("intelliPayButton").click();
                }


                function updatePaymentStatus(e){

                    e.data.response.order_id = <?php echo $order->get_id(); ?>;
                    
                    var form = document.createElement('form');
                    document.body.appendChild(form);
                    form.method = 'post';
                    form.action = "<?php echo $accept_url; ?>";
                    var input = document.createElement('input');
                    input.type = 'hidden';
                    input.name = 'data';
                    input.value = JSON.stringify(e.data.response);
                    form.appendChild(input);

                    form.submit();
                    document.body.removeChild(form);
                }

                function promptMessage(type, promptMessage){
                    //Incase we have one open already.
                    jQuery('#intelliPayCustomPrompt').remove();

                    jQuery('.woocommerce-notices-wrapper').append(  '<ul id="intelliPayCustomPrompt"  class="woocommerce-'+type+'" role="alert"><li><a href="#" onclick="openIntelliPayLightBox()">' + promptMessage + '</a></li></ul>' );
                    setTimeout(function(){
                        jQuery('#intelliPayCustomPrompt').remove();
                    },7000);
                }

                window.addEventListener(
                    'message',
                    function(e) {
                        console.log('get those events:', e);
                        
                        if (isIntelliPayEvent(e)) {
                            switch(e.data.operation){
                                case 'isready':
                                    defaultLoadOptions();
                                    break;
                                case 'approval':
                                case 'decline':
                                    updatePaymentStatus(e);
                                    break;
                                case 'closemodal':
                                    promptMessage('error', 'Payment Window closed. Reopen it by clicking here or  by clicking the button Pay Via IntelliPay below.');
                                    break;
                                default:
                                    console.log('default event not created.');

                            }   
                        }
                    }
                );
            </script>
        
        
            <?php
        }

        // Webhook Listener function
        function check_notify_response() {

            $data = json_decode(str_replace( '\"', '"', $_POST['data'] ), true);


            // Could also use $data['account'] that comes back.
            // Just being on the safe said if it was to get changed by the user.
            $order_id = $data['order_id']; 

    
        
            if ( $data['response'] === 'A' ) {
                $order = wc_get_order( $order_id );

                //a negative amount would never happen.
                if(!empty($data['fee']) && $data['fee'] > 0){

                    $item_fee = new WC_Order_Item_Fee();
                    $item_fee->set_name( "Processing Fee" ); // Name of the fee
                    $item_fee->set_amount( $data['fee'] ); // Fee amount
                    $item_fee->set_tax_class( '' ); // default for ''
                    $item_fee->set_tax_status( 'none' ); // taxable or 'none'
                    $item_fee->set_total( $data['fee'] ); // Fee amount

                    $order->add_item( $item_fee );
                    $order->calculate_totals();
                    $order->save();
                }

                $order_notes = 'payment id: ' .  $data['paymentid'];
                $order_notes .= (!empty($data['authcode'])) ? ' authcode: ' .  $data['authcode'] : '';
                $order->add_order_note( $order_notes );
                $order->payment_complete();
                wp_safe_redirect( $this->get_return_url( $order ) );
                exit();

            } else {
                $genericMessage = 'Error occurred' . "\n" . print_r( $data, true );
                $message = (!empty($data['declinereason'])) ? 'Error occurred : ' . $data['declinereason'] : $genericMessage;
                $this->cancel_order( $message, $order_id );
            }
        }

        private function cancel_order( $message = '', $order_id = false ) {

            if ( $order_id === false) {
                wc_add_notice( $message , 'error' );
                $return_url = wc_get_cart_url();

            } else {
                $order = wc_get_order( $order_id );
                $order->cancel_order( $message );
                wc_add_notice( $message , 'error' );
                $return_url = $order->get_cancel_order_url();
            }

            wp_safe_redirect( $return_url );
            exit();
        }

    }
}

add_action( 'plugins_loaded', 'init_intellipay_payment_gateway' );


function add_wc_add_intellipay_class_payment_gateway( $methods ) {
    if ( class_exists( 'WC_Payment_Gateway' ) ) {
        $methods[] = 'WC_Gateway_IntelliPay_Payment_Gateway';
    }
    return $methods;
}

add_filter( 'woocommerce_payment_gateways', 'add_wc_add_intellipay_class_payment_gateway' );
